@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: xfce-panel-plugin-iface.h 24236 2007-01-01 19:51:30Z nick $
+/*  $Id: xfce-panel-plugin-iface.h 25719 2007-05-17 08:58:02Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -51,18 +51,37 @@ typedef struct _XfcePanelPluginInterface XfcePanelPluginInterface;
  **/
 typedef void (*XfcePanelPluginFunc) (XfcePanelPlugin *plugin);
 
+/**
+ * XfcePanelPluginPreInit:
+ * @argc : number of arguments to the plugin
+ * @argv : argument array
+ *
+ * Callback function that is run in an external plugin before gtk_init(). It 
+ * should return %FALSE if the plugin is not available for whatever reason. 
+ * The function can be given as argument to one of the registration macros.
+ *
+ * The main purpose of this callback is to allow multithreaded plugins to call
+ * g_thread_init().
+ *
+ * Returns: %TRUE on success, %FALSE otherwise.
+ *
+ * See also: XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_FULL()
+ **/
+typedef gboolean (*XfcePanelPluginPreInit) (int argc, char **argv);
+
 /**
  * XfcePanelPluginCheck:
- * @screen : The #GdkScreen that the plugin should be run on.
+ * @screen : the #GdkScreen the panel is running on
  *
  * Callback function that is run before creating a plugin. It should return
- * if the plugin is not available for whatever reason. It should be given as 
- * the argument to the registration macros.
+ * %FALSE if the plugin is not available for whatever reason. The function 
+ * can be given as argument to one of the registration macros.
  *
- * Returns: %TRUE if the plugin can be started, %FALSE other wise.
+ * Returns: %TRUE if the plugin can be started, %FALSE otherwise.
  *
- * See also: XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_WITH_CHECK() and
- *           XFCE_PANEL_PLUGIN_REGISTER_INTERNAL_WITH_CHECK()
+ * See also: XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_WITH_CHECK(),
+ *           XFCE_PANEL_PLUGIN_REGISTER_INTERNAL_WITH_CHECK() and
+ *           XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_FULL()
  **/
 typedef gboolean (*XfcePanelPluginCheck) (GdkScreen *screen);
 
