@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: panel-app.c 24093 2006-12-13 19:57:17Z jasper $
+/*  $Id: panel-app.c 28348 2008-10-21 20:12:41Z olivier $
  *
  *  Copyright © 2005-2006 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -364,7 +364,11 @@ monitor_size_changed (GdkScreen *screen)
     {
         monitor = g_ptr_array_index (panel_app.monitor_list, i);
 
-        if (monitor->screen == screen)
+        /* 
+         * With xrandr 1.2, monitors can be added/removed, so need 
+         * to double check the number of monitors (bug #3620)...
+         */
+        if ((monitor->screen == screen) && (monitor->num < gdk_screen_get_n_monitors (screen)))
         {
             gdk_screen_get_monitor_geometry (screen, monitor->num, 
                                              &(monitor->geometry));
@@ -736,7 +740,6 @@ panel_app_run (int    argc,
     
     /* cleanup */
     g_free (panel_app.session_client);
-    panel_app.session_client = NULL;
 
     cleanup_panels ();
     xfce_panel_item_manager_cleanup ();
