@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: panel-dnd.h 20070 2006-02-25 22:33:57Z jasper $
+/*  $Id: panel-dnd.h 26669 2008-03-09 18:03:37Z nick $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -27,26 +27,16 @@
 enum 
 {
     TARGET_PLUGIN_NAME,
-    TARGET_PLUGIN_WIDGET,
-    TARGET_FILE
+    TARGET_PLUGIN_WIDGET
 };
 
-void panel_dnd_set_dest (GtkWidget *widget);
+void panel_dnd_set_dest_name_and_widget (GtkWidget *widget);
 
-void panel_dnd_set_widget_delete_dest (GtkWidget *widget);
+void panel_dnd_set_dest_widget (GtkWidget *widget);
 
-void panel_dnd_unset_dest (GtkWidget *widget);
+void panel_dnd_set_source_name (GtkWidget *widget);
 
-GtkWidget *panel_dnd_get_plugin_from_data (GtkSelectionData *data);
-
-
-void panel_dnd_set_name_source (GtkWidget *widget);
-
-void panel_dnd_set_widget_source (GtkWidget *widget);
-
-void panel_dnd_unset_source (GtkWidget *widget);
-
-void panel_dnd_set_widget_data (GtkSelectionData *data, GtkWidget *plugin);
+void panel_dnd_set_source_widget (GtkWidget *widget);
 
 void panel_dnd_begin_drag (GtkWidget *widget);
 
