@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: panel-item-manager.c 24080 2006-12-11 21:45:26Z jasper $
+/*  $Id: panel-item-manager.c 25755 2007-05-24 18:40:41Z nick $
  *
  *  Copyright © 2005-2006 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -217,6 +217,7 @@ new_plugin_class_from_desktop_file (const char *file)
         xfce_rc_set_group (rc, "Xfce Panel");
 
         if ((value = xfce_rc_read_entry (rc, "X-XFCE-Exec", NULL)) 
+            && g_path_is_absolute (value)
             && g_file_test (value, G_FILE_TEST_EXISTS))
 
         {
@@ -225,24 +226,27 @@ new_plugin_class_from_desktop_file (const char *file)
         }
         else if ((value = xfce_rc_read_entry (rc, "X-XFCE-Module", NULL)))
         {
-            if (g_file_test (value, G_FILE_TEST_EXISTS))
-            {
-                class = create_item_class (value, FALSE);
-                DBG ("Internal plugin: %s", value);
-            }
-            else if ((dir = xfce_rc_read_entry (rc, "X-XFCE-Module-Path", 
-                                                NULL)))
+            if ((dir = xfce_rc_read_entry (rc, "X-XFCE-Module-Path", NULL)))
             {
                 path = g_module_build_path (dir, value);
 
-                if (g_file_test (path, G_FILE_TEST_EXISTS))
+                if (g_path_is_absolute (path)
+                    && g_file_test (path, G_FILE_TEST_EXISTS))
                 {
                     class = create_item_class (path, FALSE);
+                    
                     DBG ("Internal plugin: %s", path);
                 }
 
                 g_free (path);
             }
+            else if (g_path_is_absolute (value)
+                     && g_file_test (value, G_FILE_TEST_EXISTS))
+            {
+                class = create_item_class (value, FALSE);
+                
+                DBG ("Internal plugin: %s", value);
+            }
         }
 
         if (class)
@@ -297,6 +301,13 @@ update_plugin_list (void)
     else
     {
         dirs = xfce_resource_dirs (XFCE_RESOURCE_DATA);
+
+        if (G_UNLIKELY(!dirs))
+        {
+            dirs    = g_new (char*, 2);
+            dirs[0] = g_strdup (DATADIR);
+            dirs[1] = NULL;
+        }
     }
     
     for (d = dirs; *d != NULL || !datadir_used; ++d)
@@ -343,9 +354,12 @@ update_plugin_list (void)
                 {
                     DBG (" + class \"%s\": "
                          "name=%s, comment=%s, icon=%s, external=%d, path=%s", 
-                         class->plugin_name,
-                         class->name, class->comment, class->icon,
-                         class->is_external, class->file);
+                         class->plugin_name ? class->plugin_name : "(null)",
+                         class->name        ? class->name        : "(null)", 
+                         class->comment     ? class->comment     : "(null)", 
+                         class->icon        ? class->icon        : "(null)",
+                         class->is_external, 
+                         class->file        ? class->file        : "(null)");
                 
                     g_hash_table_insert (plugin_classes, 
                                          class->plugin_name, 
