@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: panel-properties.c 25940 2007-07-30 19:22:58Z jasper $
+/*  $Id: panel-properties.c 26669 2008-03-09 18:03:37Z nick $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -939,7 +939,7 @@ void panel_init_signals (Panel *panel)
     
     g_signal_connect (panel, "move-end", G_CALLBACK (panel_move_end), NULL);
 
-    panel_dnd_set_dest (GTK_WIDGET (panel));
+    panel_dnd_set_dest_name_and_widget (GTK_WIDGET (panel));
     g_signal_connect (panel, "drag-motion", G_CALLBACK (drag_motion), NULL);
     g_signal_connect (panel, "drag-leave", G_CALLBACK (drag_leave), NULL);
 }
