@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: iconbox.c 21756 2006-05-21 12:59:15Z jasper $
+/*  $Id: iconbox.c 24235 2007-01-01 19:22:10Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -31,9 +31,11 @@
 #include <libxfcegui4/netk-window-action-menu.h>
 #include <libxfce4panel/xfce-panel-plugin.h>
 #include <libxfce4panel/xfce-panel-convenience.h>
+#include <libxfce4panel/xfce-hvbox.h>
 
 #define N_ICONBOX_CONNECTIONS  4
 #define N_ICON_CONNECTIONS     4
+#define URGENT_TIMEOUT         500
 
 typedef struct
 {
@@ -44,7 +46,6 @@ typedef struct
     int screen_changed_id;
     GtkWidget *box;
     GtkWidget *handle;
-    GtkWidget *handle2;
     GtkWidget *iconbox;
 
     GSList *iconlist;
@@ -52,6 +53,8 @@ typedef struct
 
     int icon_size;
     guint only_hidden:1;
+    guint all_workspaces:1;
+    guint expand:1;
 }
 Iconbox;
 
@@ -67,6 +70,7 @@ typedef struct
     GtkWidget *button;
     GtkWidget *image;
     gboolean was_minimized;
+    guint urgent_id;
 }
 Icon;
 
@@ -111,8 +115,10 @@ update_visibility (Icon *icon, NetkWorkspace *optional_active_ws)
                 (netk_window_get_screen (icon->window));
     }
     
-    if (ws == netk_window_get_workspace (icon->window) ||
-        netk_window_is_sticky (icon->window))
+    if (icon->ib->all_workspaces 
+        || netk_window_is_sticky (icon->window) 
+        || ws == netk_window_get_workspace (icon->window) 
+        || netk_window_or_transient_demands_attention (icon->window))
     {
         gtk_widget_show (icon->button);
     }
@@ -167,7 +173,7 @@ icon_update_image (Icon *icon)
             xfce_scaled_image_set_from_pixbuf (XFCE_SCALED_IMAGE (icon->image), 
                                                scaled);
 
-            g_object_unref (scaled);
+            g_object_unref (G_OBJECT (scaled));
 
             icon->was_minimized = TRUE;
         }
@@ -183,6 +189,75 @@ icon_update_image (Icon *icon)
     update_visibility (icon, NULL);
 }
 
+/* blinking */
+static void
+update_blink (Icon *icon, gboolean blink)
+{
+    GtkStyle   *style;
+    GtkRcStyle *mod;
+    GdkColor    c;
+
+    style = gtk_widget_get_style (icon->button);
+    mod   = gtk_widget_get_modifier_style (icon->button);
+    c     = style->bg[GTK_STATE_SELECTED];
+
+    if (blink)
+    {
+	gtk_button_set_relief (GTK_BUTTON (icon->button), GTK_RELIEF_NORMAL);
+	
+	if(mod->color_flags[GTK_STATE_NORMAL] & GTK_RC_BG)
+        {
+            mod->color_flags[GTK_STATE_NORMAL] &= ~(GTK_RC_BG);
+            gtk_widget_modify_style(icon->button, mod);
+        }
+        else
+        {
+            mod->color_flags[GTK_STATE_NORMAL] |= GTK_RC_BG;
+            mod->bg[GTK_STATE_NORMAL] = c;
+            gtk_widget_modify_style(icon->button, mod);
+        }
+    }
+    else
+    {
+        gtk_button_set_relief (GTK_BUTTON (icon->button), GTK_RELIEF_NONE);
+	mod->color_flags[GTK_STATE_NORMAL] &= ~(GTK_RC_BG);
+	gtk_widget_modify_style(icon->button, mod);
+    }
+}
+
+static gboolean
+urgent_timeout (Icon *icon)
+{
+    update_visibility (icon, NULL);
+    update_blink (icon, TRUE);
+    return TRUE;;
+}
+
+static void
+queue_urgent_timeout (Icon *icon)
+{
+    if (icon->urgent_id == 0)
+    {
+        icon->urgent_id = 
+            g_timeout_add (URGENT_TIMEOUT, (GSourceFunc)urgent_timeout, icon);
+    }
+}
+
+static void
+unqueue_urgent_timeout (Icon *icon)
+{
+    if (icon->urgent_id > 0)
+    {
+        g_source_remove (icon->urgent_id);
+        icon->urgent_id = 0;
+        if (icon->button)
+        {
+            update_blink (icon, FALSE);
+            update_visibility (icon, NULL);
+        }
+    }
+}
+
 /* callbacks */
 static gboolean
 icon_button_pressed (GtkWidget *button, GdkEventButton *ev, gpointer data)
@@ -192,9 +267,24 @@ icon_button_pressed (GtkWidget *button, GdkEventButton *ev, gpointer data)
     if (ev->button == 1)
     {
         if (netk_window_is_active (icon->window))
+        {
             netk_window_minimize (icon->window);
+        }
         else
+        {
+            NetkScreen    *scr;
+            NetkWorkspace *aws, *ws;
+
+            scr = netk_window_get_screen (icon->window);
+            aws = netk_screen_get_active_workspace (scr);
+            ws  = netk_window_get_workspace (icon->window);
+
+            if (aws != ws)
+            {
+                netk_workspace_activate (ws);
+            }
             netk_window_activate (icon->window);
+        }
 
         return TRUE;
     }
@@ -231,7 +321,21 @@ icon_state_changed (NetkWindow *window, NetkWindowState changed_mask,
 {
     Icon *icon = (Icon *)data;
 
-    if (changed_mask & NETK_WINDOW_STATE_MINIMIZED)
+    if (changed_mask & NETK_WINDOW_STATE_DEMANDS_ATTENTION
+        || changed_mask & NETK_WINDOW_STATE_URGENT)
+    {
+        if (netk_window_or_transient_demands_attention (window))
+        {
+            queue_urgent_timeout (icon);
+        }
+        else
+        {
+            unqueue_urgent_timeout (icon);
+        }
+    }
+
+    if (changed_mask & NETK_WINDOW_STATE_MINIMIZED
+        || changed_mask & NETK_WINDOW_STATE_SKIP_TASKLIST)
     {
         update_visibility (icon, NULL);
 
@@ -253,13 +357,15 @@ icon_icon_changed (NetkWindow *window, gpointer data)
     Icon *icon = (Icon *)data;
     
     if (icon->pb)
-        g_object_unref (icon->pb);
+        g_object_unref (G_OBJECT (icon->pb));
 
     icon->pb = netk_window_get_icon (icon->window);
 
     if (icon->pb)
-        g_object_ref (icon->pb);
+        g_object_ref (G_OBJECT (icon->pb));
     
+    /* make sure the icon is actually updated */
+    icon->was_minimized = !netk_window_is_minimized (icon->window);
     icon_update_image (icon);
 }
 
@@ -268,6 +374,8 @@ icon_destroy (Icon *icon)
 {
     int i;
 
+    unqueue_urgent_timeout (icon);
+    
     for (i = 0; i < N_ICON_CONNECTIONS; i++)
     {
         if (icon->connections[i])
@@ -277,15 +385,15 @@ icon_destroy (Icon *icon)
     }
     
     if (icon->pb)
-        g_object_unref (icon->pb);
+        g_object_unref (G_OBJECT (icon->pb));
     
-    g_free (icon);
+    panel_slice_free (Icon, icon);
 }
 
 static Icon *
 icon_new (NetkWindow *window, Iconbox *ib)
 {
-    Icon *icon = g_new0 (Icon, 1);
+    Icon *icon = panel_slice_new0 (Icon);
     int i = 0;
 
     icon->ib = ib;
@@ -306,7 +414,7 @@ icon_new (NetkWindow *window, Iconbox *ib)
     {
         xfce_scaled_image_set_from_pixbuf (XFCE_SCALED_IMAGE (icon->image), 
                                            icon->pb);
-        g_object_ref (icon->pb);
+        g_object_ref (G_OBJECT (icon->pb));
     }
     
     icon->connections[i++] = 
@@ -385,6 +493,11 @@ iconbox_window_opened (NetkScreen *screen, NetkWindow *window, gpointer data)
     ib->iconlist = g_slist_append (ib->iconlist, icon);
 
     gtk_box_pack_start (GTK_BOX (ib->iconbox), icon->button, FALSE, FALSE, 0);
+
+    if (netk_window_or_transient_demands_attention (window))
+    {
+        queue_urgent_timeout (icon);
+    }
 }
 
 static void
@@ -400,6 +513,7 @@ iconbox_window_closed (NetkScreen *screen, NetkWindow *window, gpointer data)
         if (window == icon->window)
         {
             gtk_widget_destroy (icon->button);
+            icon->button = NULL;
             icon_destroy (icon);
 
             ib->iconlist = g_slist_delete_link (ib->iconlist, l);
@@ -480,9 +594,9 @@ cleanup_iconbox (Iconbox *ib)
 {
     cleanup_icons (ib);
 
-    g_object_unref (ib->icon_tooltips);
+    g_object_unref (G_OBJECT (ib->icon_tooltips));
     
-    g_free (ib);
+    panel_slice_free (Iconbox, ib);
 }
 
 
@@ -499,49 +613,13 @@ XFCE_PANEL_PLUGIN_REGISTER_INTERNAL (iconbox_construct);
 
 static void
 iconbox_orientation_changed (XfcePanelPlugin *plugin, 
-                              GtkOrientation orientation, 
-                              Iconbox *iconbox)
+                             GtkOrientation orientation, 
+                             Iconbox *iconbox)
 {
-    GtkWidget *box;
-    GList *children, *l;
-
-    /* iconbox */
-    box = (orientation == GTK_ORIENTATION_HORIZONTAL) ?
-            gtk_hbox_new (TRUE, 0) : gtk_vbox_new (TRUE, 0);
-    gtk_container_set_reallocate_redraws (GTK_CONTAINER (box), TRUE);
-    gtk_widget_show (box);
-
-    children = gtk_container_get_children (GTK_CONTAINER (iconbox->iconbox));
-
-    for (l = children; l != NULL; l = l->next)
-    {
-        gtk_widget_reparent (GTK_WIDGET (l->data), box);
-    }
-    
-    g_list_free (children);
-
-    iconbox->iconbox = box;
-    
-    box = (orientation == GTK_ORIENTATION_HORIZONTAL) ?
-            gtk_hbox_new (FALSE, 0) : gtk_vbox_new (FALSE, 0);
-    gtk_widget_show (box);
-    
-    gtk_widget_reparent (iconbox->handle, box);
-    gtk_box_set_child_packing (GTK_BOX (box), iconbox->handle,
-                               FALSE, FALSE, 0, GTK_PACK_START);
-    
-    gtk_box_pack_start (GTK_BOX (box), iconbox->iconbox, FALSE, FALSE, 0);
-    
-    gtk_widget_reparent (iconbox->handle2, box);
-    gtk_box_set_child_packing (GTK_BOX (box), iconbox->handle2,
-                               FALSE, FALSE, 0, GTK_PACK_START);
-    
-    gtk_widget_destroy (iconbox->box);
-    iconbox->box = box;
-    gtk_container_add (GTK_CONTAINER (plugin), box);
+    xfce_hvbox_set_orientation (XFCE_HVBOX(iconbox->box), orientation);
+    xfce_hvbox_set_orientation (XFCE_HVBOX(iconbox->iconbox), orientation);
 
     gtk_widget_queue_draw (iconbox->handle);
-    gtk_widget_queue_draw (iconbox->handle2);
 }
 
 static gboolean 
@@ -594,6 +672,8 @@ iconbox_read_rc_file (XfcePanelPlugin *plugin, Iconbox *iconbox)
     char *file;
     XfceRc *rc;
     int only_hidden = 0;
+    int all_workspaces = 0;
+    int expand = 1;
     
     if ((file = xfce_panel_plugin_lookup_rc_file (plugin)) != NULL)
     {
@@ -603,12 +683,15 @@ iconbox_read_rc_file (XfcePanelPlugin *plugin, Iconbox *iconbox)
         if (rc != NULL)
         {
             only_hidden = xfce_rc_read_int_entry (rc, "only_hidden", 0);
-            
+            all_workspaces = xfce_rc_read_int_entry (rc, "all_workspaces", 0);
+            expand = xfce_rc_read_int_entry (rc, "expand", 1);
             xfce_rc_close (rc);
         }
     }
 
     iconbox->only_hidden = (only_hidden == 1);
+    iconbox->all_workspaces = (all_workspaces == 1);
+    iconbox->expand = (expand != 0);
 }
 
 static void
@@ -627,6 +710,8 @@ iconbox_write_rc_file (XfcePanelPlugin *plugin, Iconbox *iconbox)
         return;
     
     xfce_rc_write_int_entry (rc, "only_hidden", iconbox->only_hidden);
+    xfce_rc_write_int_entry (rc, "all_workspaces", iconbox->all_workspaces);
+    xfce_rc_write_int_entry (rc, "expand", iconbox->expand);
 
     xfce_rc_close (rc);
 }
@@ -639,17 +724,33 @@ handle_expose (GtkWidget *widget, GdkEventExpose *ev, Iconbox *iconbox)
     {
         GtkAllocation *allocation = &(widget->allocation);
         int x, y, w, h;
+        gboolean horizontal;
 
-        x = allocation->x + widget->style->xthickness;
-        y = allocation->y + widget->style->ythickness;
-        w = allocation->width - 2 * widget->style->xthickness;
-        h = allocation->height - 2 * widget->style->ythickness;
+        horizontal = (xfce_panel_plugin_get_orientation (iconbox->plugin) 
+                      == GTK_ORIENTATION_HORIZONTAL);
 
-        gtk_paint_box (widget->style, widget->window, 
-                       GTK_WIDGET_STATE (widget),
-                       GTK_SHADOW_OUT,
-                       &(ev->area), widget, "xfce-panel",
-                       x, y, w, h);        
+        x = allocation->x;
+        y = allocation->y;
+        w = allocation->width;
+        h = allocation->height;
+        
+        if (horizontal)
+        {
+            y += widget->style->ythickness;
+            h -= 2 * widget->style->ythickness;
+        }
+        else
+        {
+            x += widget->style->xthickness;
+            w -= 2 * widget->style->xthickness;
+        }
+
+        gtk_paint_handle (widget->style, widget->window,
+                          GTK_WIDGET_STATE (widget), GTK_SHADOW_NONE, 
+                          &(ev->area), widget, "handlebox",
+                          x, y, w, h,
+                          horizontal ? GTK_ORIENTATION_VERTICAL :
+                                       GTK_ORIENTATION_HORIZONTAL);
 
         return TRUE;
     }
@@ -660,9 +761,13 @@ handle_expose (GtkWidget *widget, GdkEventExpose *ev, Iconbox *iconbox)
 static void
 iconbox_screen_changed (GtkWidget *plugin, GdkScreen *screen, Iconbox *ib)
 {
+    screen = gtk_widget_get_screen (plugin);
+
     if (!screen)
         return;
 
+    gtk_container_foreach (GTK_CONTAINER (ib->iconbox), 
+                           (GtkCallback) gtk_widget_destroy, NULL);
     cleanup_icons (ib);
     
     ib->netk_screen = netk_screen_get (gdk_screen_get_number (screen));
@@ -673,7 +778,40 @@ iconbox_screen_changed (GtkWidget *plugin, GdkScreen *screen, Iconbox *ib)
 static void
 iconbox_construct (XfcePanelPlugin *plugin)
 {
-    Iconbox *iconbox = g_new0 (Iconbox, 1);
+    Iconbox *iconbox = panel_slice_new0 (Iconbox);
+
+    iconbox->plugin = plugin;
+    
+    iconbox_read_rc_file (plugin, iconbox);
+
+    xfce_panel_plugin_set_expand (plugin, iconbox->expand);
+    
+    iconbox->box = xfce_hvbox_new (xfce_panel_plugin_get_orientation (plugin), 
+                                   FALSE, 0);
+    gtk_container_set_reallocate_redraws (GTK_CONTAINER (iconbox->box), TRUE);
+    gtk_widget_show (iconbox->box);
+    gtk_container_add (GTK_CONTAINER (plugin), iconbox->box);
+
+    iconbox->handle = gtk_alignment_new (0, 0, 0, 0);
+    gtk_widget_set_size_request (iconbox->handle, 8, 8);
+    gtk_widget_show (iconbox->handle);
+    gtk_box_pack_start (GTK_BOX (iconbox->box), iconbox->handle, 
+                        FALSE, FALSE, 0);
+
+    xfce_panel_plugin_add_action_widget (plugin, iconbox->handle);
+
+    g_signal_connect (iconbox->handle, "expose-event", 
+                      G_CALLBACK (handle_expose), iconbox);
+    
+    iconbox->iconbox = 
+        xfce_hvbox_new (xfce_panel_plugin_get_orientation (plugin), FALSE, 0);
+    gtk_widget_show (iconbox->iconbox);
+    gtk_box_pack_start (GTK_BOX (iconbox->box), iconbox->iconbox, 
+                        FALSE, FALSE, 0);
+    
+    iconbox->icon_tooltips = gtk_tooltips_new ();
+    g_object_ref (G_OBJECT (iconbox->icon_tooltips));
+    gtk_object_sink (GTK_OBJECT (iconbox->icon_tooltips));
 
     g_signal_connect (plugin, "orientation-changed", 
                       G_CALLBACK (iconbox_orientation_changed), iconbox);
@@ -691,52 +829,6 @@ iconbox_construct (XfcePanelPlugin *plugin)
     g_signal_connect (plugin, "configure-plugin", 
                       G_CALLBACK (iconbox_properties_dialog), iconbox);
 
-    xfce_panel_plugin_set_expand (plugin, TRUE);
-    
-    iconbox_read_rc_file (plugin, iconbox);
-
-    iconbox->plugin = plugin;
-    
-    iconbox->box = (xfce_panel_plugin_get_orientation (plugin) == 
-                        GTK_ORIENTATION_HORIZONTAL) ?
-                    gtk_hbox_new (FALSE, 0) : gtk_vbox_new (FALSE, 0);
-    gtk_container_set_reallocate_redraws (GTK_CONTAINER (iconbox->box), TRUE);
-    gtk_widget_show (iconbox->box);
-    gtk_container_add (GTK_CONTAINER (plugin), iconbox->box);
-
-    iconbox->handle = gtk_alignment_new (0, 0, 0, 0);
-    gtk_widget_set_size_request (iconbox->handle, 6, 6);
-    gtk_widget_show (iconbox->handle);
-    gtk_box_pack_start (GTK_BOX (iconbox->box), iconbox->handle, 
-                        FALSE, FALSE, 0);
-
-    xfce_panel_plugin_add_action_widget (plugin, iconbox->handle);
-
-    g_signal_connect (iconbox->handle, "expose-event", 
-                      G_CALLBACK (handle_expose), iconbox);
-    
-    iconbox->iconbox = (xfce_panel_plugin_get_orientation (plugin) == 
-                        GTK_ORIENTATION_HORIZONTAL) ?
-                       gtk_hbox_new (TRUE, 0) : gtk_vbox_new (TRUE, 0);
-    gtk_widget_show (iconbox->iconbox);
-    gtk_box_pack_start (GTK_BOX (iconbox->box), iconbox->iconbox, 
-                        FALSE, FALSE, 0);
-    
-    iconbox->handle2 = gtk_alignment_new (0, 0, 0, 0);
-    gtk_widget_set_size_request (iconbox->handle2, 6, 6);
-    gtk_widget_show (iconbox->handle2);
-    gtk_box_pack_start (GTK_BOX (iconbox->box), iconbox->handle2, 
-                        FALSE, FALSE, 0);
-
-    xfce_panel_plugin_add_action_widget (plugin, iconbox->handle2);
-
-    g_signal_connect (iconbox->handle2, "expose-event", 
-                      G_CALLBACK (handle_expose), iconbox);
-    
-    iconbox->icon_tooltips = gtk_tooltips_new ();
-    g_object_ref (iconbox->icon_tooltips);
-    gtk_object_sink (GTK_OBJECT (iconbox->icon_tooltips));
-
     iconbox->screen_changed_id = 
         g_signal_connect (plugin, "screen-changed", 
                           G_CALLBACK (iconbox_screen_changed), iconbox);
@@ -765,6 +857,29 @@ only_hidden_toggled (GtkToggleButton *tb, Iconbox *ib)
     }
 }
 
+static void
+all_workspaces_toggled (GtkToggleButton *tb, Iconbox *ib)
+{
+    GSList *l;
+    
+    ib->all_workspaces = gtk_toggle_button_get_active (tb);
+
+    for (l = ib->iconlist; l != NULL; l = l->next)
+    {
+        Icon *icon = l->data;        
+
+        update_visibility (icon, NULL);
+    }
+}
+
+static void
+expand_toggled (GtkToggleButton *tb, Iconbox *ib)
+{
+    ib->expand = gtk_toggle_button_get_active (tb);
+
+    xfce_panel_plugin_set_expand (ib->plugin, ib->expand);
+}
+
 static void
 iconbox_dialog_response (GtkWidget *dlg, int reponse, 
                          Iconbox *ib)
@@ -783,12 +898,14 @@ iconbox_properties_dialog (XfcePanelPlugin *plugin, Iconbox *iconbox)
 
     xfce_panel_plugin_block_menu (plugin);
     
-    dlg = xfce_titled_dialog_new_with_buttons (_("Icon Box"),
-                GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
+    dlg = xfce_titled_dialog_new_with_buttons (_("Icon Box"), NULL,
                 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
                 GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
                 NULL);
 
+    gtk_window_set_screen (GTK_WINDOW (dlg), 
+                           gtk_widget_get_screen (GTK_WIDGET (plugin)));
+
     g_object_set_data (G_OBJECT (plugin), "dialog", dlg);
 
     gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
@@ -813,7 +930,24 @@ iconbox_properties_dialog (XfcePanelPlugin *plugin, Iconbox *iconbox)
                                   iconbox->only_hidden);
     g_signal_connect (cb, "toggled", G_CALLBACK (only_hidden_toggled),
                       iconbox);
-  
+    
+    cb = gtk_check_button_new_with_mnemonic (
+                _("Show applications of all workspaces"));
+    gtk_widget_show (cb);
+    gtk_box_pack_start (GTK_BOX (vbox), cb, FALSE, FALSE, 0);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
+                                  iconbox->all_workspaces);
+    g_signal_connect (cb, "toggled", G_CALLBACK (all_workspaces_toggled),
+                      iconbox);
+    
+    cb = gtk_check_button_new_with_mnemonic (_("Use all available space"));
+    gtk_widget_show (cb);
+    gtk_box_pack_start (GTK_BOX (vbox), cb, FALSE, FALSE, 0);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
+                                  iconbox->expand);
+    g_signal_connect (cb, "toggled", G_CALLBACK (expand_toggled),
+                      iconbox);
+
     gtk_widget_show (dlg);
 }
 
